/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file demonstrates how to implement a simple chat server. The
 * ServeChatClientInput class implements a thread that essentially
 * waits for input from a client and then sticks it into the input
 * queue of the dispatcher.
 */

/*
 * TODO:
 * - get rid of the BufferedReader()s since they tend to not react
 *   if, for example, the server goes down.
 */

import java.net.Socket;
import java.io.Reader;
import java.io.Writer;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import java.util.concurrent.BlockingQueue;

public class ServeChatClientInput
    implements Runnable
{
    private Socket socket;
    private BufferedReader in;
    private ChatDispatcher dispatcher;
    private BlockingQueue<String> queue;

    public ServeChatClientInput(Socket s, ChatDispatcher d,
				BlockingQueue<String> q)
	throws IOException
    {
	socket = s;
	dispatcher = d;
	queue = q;
	in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
    }
    
    public void run()
    {
	try {
	    while (! Thread.currentThread().isInterrupted()) {
		String s = in.readLine();
		if (s == null) break;
		queue.offer(ChatServer.getClientName(socket) + " " + s);
	    }
	} catch (IOException e) {
	    System.err.println(e);
	} finally {
	    try {
		socket.close();
	    } catch (IOException e) {
		System.err.println(e);
	    }
	    dispatcher.removeClient(socket);
	}
    }
}
